@@ -5,9 +5,10 @@ from collections import OrderedDict
 from tensorflow.keras.models import load_model
 from pkg_resources import resource_filename
 from distutils.spawn import find_executable
+from tcn import TCN, tcn_full_summary
 
 from astrorapid.process_light_curves import read_multiple_light_curves
-from astrorapid.prepare_arrays import PrepareInputArrays
+from astrorapid.prepare_input import PrepareInputArrays
 
 try:
     import matplotlib.pyplot as plt
@@ -27,40 +28,48 @@ except ImportError:
 CLASS_COLOR = {'Pre-explosion': 'grey', 'SNIa-norm': 'tab:green', 'SNIbc': 'tab:orange', 'SNII': 'tab:blue',
                'SNIa-91bg': 'tab:red', 'SNIa-x': 'tab:purple', 'point-Ia': 'tab:brown', 'Kilonova': '#aaffc3',
                'SLSN-I': 'tab:olive', 'PISN': 'tab:cyan', 'ILOT': '#FF1493', 'CART': 'navy', 'TDE': 'tab:pink',
-               'AGN': 'bisque'}
+               'AGN': 'bisque',
+               'Ia': 'tab:green', 'SLSN': 'tab:olive', 'II': 'tab:blue', 'IIn': 'tab:brown', 'Ibc': 'tab:orange'}
 PB_COLOR = {'u': 'tab:blue', 'g': 'tab:blue', 'r': 'tab:orange', 'i': 'm', 'z': 'k', 'Y': 'y'}
-PB_MARKER = {'g': 'o', 'r': 's'}
-PB_ALPHA = {'g': 0.3, 'r': 1.}
 SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
 
 
 class Classify(object):
-    def __init__(self, known_redshift=True, model_filepath='', passbands=('g', 'r'),
+    def __init__(self, model_name='ZTF_known_redshift', model_filepath='', known_redshift=True, passbands=('g', 'r'),
                  class_names=('Pre-explosion', 'SNIa-norm', 'SNIbc', 'SNII', 'SNIa-91bg', 'SNIa-x', 'Kilonova', 'SLSN-I', 'TDE'),
                  nobs=50, mintime=-70, maxtime=80, timestep=3.0, bcut=False, zcut=None, graph=None, model=None):
         """ Takes a list of photometric information and classifies light curves as a function of time
 
         Parameters
         ----------
-        known_redshift : bool
-            Different model to be used if redshift is not known.
+        model_name : str
+            The name of the pretrained model to use.
+            Choose one of: 'PS1_known_redshift_Ia_CC_SLSN', 'PS1_known_redshift_Ia_II_IIn_Ibc_SLSN', 'ZTF_known_redshift', 'ZTF_unknown_redshift'.
         model_filepath : str
-            Optional argument. The model is taken from the pre-trained model ZTF model if not specified.
+            Optional argument. If not specified, the pre-trained model specified in the model_name argument is used.
+        known_redshift : bool
+            Optional argument. Only needed if you are not using a default model and have specified model_filepath.
         passbands : tuple
-            Optional argument. A tuple listing each passband. E.g. ('g', 'r').
+            Optional argument. Only needed if you are not using a default model and have specified model_filepath.
+            A tuple listing each passband. E.g. ('g', 'r').
         class_names : tuple
+            Optional argument. Only needed if you are not using a default model and have specified model_filepath.
             List of class names that the model has been trained on. Note that this must be in the same order
             as used in training for the model specified in the argument model_filepath.
             If you are using the default model, leave this argument out.
         nobs : int
+            Optional argument. Only needed if you are not using a default model and have specified model_filepath.
             Number of points to use in interpolation of light curve between mintime and maxtime.
             Do not change this argument unless you are using your own model_filepath
             and have changed this during training.
         mintime : int
+            Optional argument. Only needed if you are not using a default model and have specified model_filepath.
             Days from trigger (minimum) to extract from light curve.
         maxtime : int
+            Optional argument. Only needed if you are not using a default model and have specified model_filepath.
             Days from trigger (maximum) to extract from light curve.
         timestep : float
+            Optional argument. Only needed if you are not using a default model and have specified model_filepath.
             Time-step between interpolated points in light curve.
             Do not change this argument unless you are using your own model_filepath
             and have changed this during training.
@@ -80,42 +89,107 @@ class Classify(object):
             This would have been created with keras' load_model function e.g. model = load_model('keras_model.hdf5')
 
         """
-        self.known_redshift = known_redshift
-        self.passbands = passbands
+
         self.bcut = bcut
         self.zcut = zcut
-        self.class_names = class_names
-        self.nobs = nobs
-        self.mintime = mintime
-        self.maxtime = maxtime
-        self.timestep = timestep
-
-        if self.known_redshift:
-            self.contextual_info = ('redshift',)
-            filename = 'keras_model_with_redshift.hdf5'
-        else:
-            self.contextual_info = ()
-            filename = 'keras_model_no_redshift.hdf5'
 
         if model_filepath != '' and os.path.exists(model_filepath):
             self.model_filepath = model_filepath
+            self.contextual_info = ['redshift',] if known_redshift else []
+            self.known_redshift = known_redshift
+            self.passbands = passbands
+            self.class_names = class_names
+            self.nobs = nobs
+            self.mintime = mintime
+            self.maxtime = maxtime
+            self.timestep = timestep
         else:
             if not (model_filepath == '' and os.path.exists(model_filepath)):
-                print("Invalid keras model. Using default model...")
-            self.model_filepath = os.path.join(SCRIPT_DIR, filename)
-            if not os.path.exists(self.model_filepath):
-                self.model_filepath = resource_filename(__name__, 'keras_model_with_redshift.hdf5')
+                print("Invalid keras model. Using default model set by model_name.")
+            if model_name == 'ZTF_known_redshift':
+                self.model_filepath = os.path.join(SCRIPT_DIR, 'ZTF_known_redshift.hdf5')
+                self.contextual_info = ['redshift',]
+                self.known_redshift = True
+                self.passbands=('g', 'r')
+                self.class_names=('Pre-explosion', 'SNIa-norm', 'SNIbc', 'SNII', 'SNIa-91bg', 'SNIa-x', 'Kilonova', 'SLSN-I', 'TDE')
+                self.nobs = 50
+                self.mintime = -70
+                self.maxtime = 80
+                self.timestep = 3.0
+            elif model_name == 'ZTF_unknown_redshift':
+                self.model_filepath = os.path.join(SCRIPT_DIR, 'ZTF_unknown_redshift.hdf5')
+                self.contextual_info = []
+                self.known_redshift = False
+                self.passbands = ('g', 'r')
+                self.class_names = ('Pre-explosion', 'SNIa-norm', 'SNIbc', 'SNII', 'SNIa-91bg', 'SNIa-x', 'Kilonova', 'SLSN-I', 'TDE')
+                self.nobs = 50
+                self.mintime = -70
+                self.maxtime = 80
+                self.timestep = 3.0
+            elif model_name == 'PS1_known_redshift_Ia_CC_SLSN':
+                self.model_filepath = os.path.join(SCRIPT_DIR, 'PS1_known_redshift_Ia_CC_SLSN.hdf5')
+                self.contextual_info = ['redshift',]
+                self.known_redshift = True
+                self.passbands = ('g', 'r', 'i', 'z')
+                self.class_names = ('Pre-explosion', 'SLSN', 'CC', 'Ia')
+                self.nobs = 50
+                self.mintime = -70
+                self.maxtime = 80
+                self.timestep = 3.0
+            elif model_name == 'PS1_known_redshift_Ia_II_IIn_Ibc_SLSN':
+                self.model_filepath = os.path.join(SCRIPT_DIR, 'PS1_known_redshift_Ia_II_IIn_Ibc_SLSN.hdf5')
+                self.contextual_info = ['redshift',]
+                self.known_redshift = True
+                self.passbands = ('g', 'r', 'i', 'z')
+                self.class_names = ('Pre-explosion', 'SLSN', 'II', 'IIn', 'Ia', 'Ibc')
+                self.nobs = 50
+                self.mintime = -70
+                self.maxtime = 80
+                self.timestep = 3.0
+            elif model_name == 'PS1_unknown_redshift_Ia_CC_SLSN':
+                self.model_filepath = os.path.join(SCRIPT_DIR, 'PS1_unknown_redshift_Ia_CC_SLSN.hdf5')
+                self.contextual_info = []
+                self.known_redshift = False
+                self.passbands = ('g', 'r', 'i', 'z')
+                self.class_names = ('Pre-explosion', 'SLSN', 'II', 'IIn', 'Ia', 'Ibc')
+                self.nobs = 50
+                self.mintime = -70
+                self.maxtime = 80
+                self.timestep = 3.0
+            else:
+                raise Exception("Invalid model_name. Select a valid model_name from the following "
+                                "('PS1_known_redshift', 'PS1_unknown_redshift', "
+                                "'ZTF_known_redshift', 'ZTF_unknown_redshift'), "
+                                "or set model_filepath to a valid path to your own trained model.")
 
         print(self.model_filepath)
         self.graph = graph
         if graph is not None and model is not None:
             self.model = model
         else:
-            self.model = load_model(self.model_filepath)
+            self.model = load_model(self.model_filepath, custom_objects={'TCN': TCN})
 
-    def process_light_curves(self, light_curves):
+    def process_light_curves(self, light_curves, other_meta_data=None):
+        """
+
+        Parameters
+        ----------
+        light_curves: list of tuples
+            Each tuple in the list is of the form: (mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv)
+            for each transient.
+        other_meta_data: list of dictionaries or None
+            Each dictionary in the list contains any additional meta data to be used as contextual_info
+            when classifying (if the classifier being used was trained on the specified contextual_info).
+            E.g. other_meta_data = [{'hosttype': 3, 'host_dist': 200}, {'hosttype': 2, 'host_dist': 150},]
+
+
+        Returns
+        -------
+
+        """
         processed_lightcurves = read_multiple_light_curves(light_curves, known_redshift=self.known_redshift,
-                                                           training_set_parameters=None)
+                                                           training_set_parameters=None,
+                                                           other_meta_data=other_meta_data)
         prepareinputarrays = PrepareInputArrays(self.passbands, self.contextual_info, self.bcut, self.zcut,
                                                 self.nobs, self.mintime, self.maxtime, self.timestep)
         X, orig_lc, timesX, objids_list, trigger_mjds = prepareinputarrays.prepare_input_arrays(processed_lightcurves)
@@ -152,7 +226,7 @@ class Classify(object):
                 assert isinstance(redshift, numbers.Number)
                 assert np.isfinite(redshift)
 
-    def get_predictions(self, light_curves, return_predictions_at_obstime=False, return_objids=False):
+    def get_predictions(self, light_curves, other_meta_data=None, return_predictions_at_obstime=False, return_objids=False):
         """ Return the classification accuracies as a function of time for each class
 
         Parameters
@@ -183,6 +257,11 @@ class Classify(object):
 
                 mwebv: Milky way extinction.
 
+        other_meta_data: list of dictionaries or None
+            Each dictionary in the list contains any additional meta data to be used as contextual_info
+            when classifying (if the classifier being used was trained on the specified contextual_info).
+            E.g. other_meta_data = [{'hosttype': 3, 'host_dist': 200}, {'hosttype': 2, 'host_dist': 150},]
+
         return_predictions_at_obstime: bool
             Return the predictions at the observation times instead of at the 50 interpolated timesteps.
         return_objids : bool, optional
@@ -204,7 +283,8 @@ class Classify(object):
 
         self._do_error_checks(light_curves)
 
-        self.X, self.orig_lc, self.timesX, self.objids, self.trigger_mjds = self.process_light_curves(light_curves)
+        self.X, self.orig_lc, self.timesX, self.objids, self.trigger_mjds = self.process_light_curves(light_curves,
+                                                                                                      other_meta_data=other_meta_data)
         nobjects = len(self.objids)
 
         if nobjects == 0:
@@ -251,7 +331,7 @@ class Classify(object):
         return y_predict, time_steps
 
     def plot_light_curves_and_classifications(self, indexes_to_plot=None, step=True, use_interp_flux=False, figdir='.',
-                                              plot_matrix_input=False, light_curves=None):
+                                              plot_matrix_input=False, light_curves=None, show_plot=False):
         """
         Plot light curve (top panel) and classifications (bottom panel) vs time.
 
@@ -275,6 +355,8 @@ class Classify(object):
             (mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv).
             Here, mjd, flux, fluxerr, passband, and photflag are arrays.
             ra, dec, objid, redshift, and mwebv are floats
+        show_plot : bool
+            Run plt.show() after each object plotted.
 
         """
 
@@ -305,7 +387,7 @@ class Classify(object):
             for pbidx, pb in enumerate(used_passbands):
                 pbmask = lc_data['passband'] == pb
                 ax1.errorbar(lc_data[pbmask]['time'], lc_data[pbmask]['flux'],
-                             yerr=lc_data[pbmask]['fluxErr'], fmt=PB_MARKER[pb], label=pb,
+                             yerr=lc_data[pbmask]['fluxErr'], fmt='o', label=pb,
                              c=PB_COLOR[pb], lw=3, markersize=10, alpha=alpha_observations)
                 if plot_matrix_input:
                     ax1.plot(self.timesX[idx][:argmax], self.X[idx][:, pbidx][:argmax], c=PB_COLOR[pb], lw=3)
@@ -344,6 +426,8 @@ class Classify(object):
             savename = 'classification_vs_time_{}{}{}.pdf'.format(self.objids[idx], '_step' if step else '', '_no_interp' if not use_interp_flux else '')
             plt.savefig(os.path.join(figdir, savename))
             # plt.savefig("{}.png".format(savename))
+            if show_plot:
+                plt.show()
             plt.close()
 
         return self.orig_lc, self.timesX, self.y_predict
@@ -410,7 +494,7 @@ class Classify(object):
             def animate(i):
                 for pbidx, pb in enumerate(used_passbands):
                     ax1.plot(self.timesX[idx][:argmax][:int(i + 1)], self.X[idx][:, pbidx][:argmax][:int(i + 1)],
-                             label=pb, c=PB_COLOR[pb], lw=3)  # , markersize=10, marker=MARKPB[pb])
+                             label=pb, c=PB_COLOR[pb], lw=3)  # , markersize=10, marker='.')
 
                 for classnum, classname in enumerate(self.class_names):
                     ax2.plot(self.timesX[idx][:argmax][:int(i + 1)],
@@ -500,7 +584,7 @@ class Classify(object):
             def animate(i):
                 for pbidx, pb in enumerate(used_passbands):
                     # ax1.plot(self.timesX[idx][:argmax][:int(i + 1)], self.X[idx][:, pbidx][:argmax][:int(i + 1)],
-                    #          label=pb, c=PB_COLOR[pb], lw=3)  # , markersize=10, marker=MARKPB[pb])
+                    #          label=pb, c=PB_COLOR[pb], lw=3)  # , markersize=10, marker='.')
 
                     # dea = [self.orig_lc[idx][pb]['time'] < self.timesX[idx][:argmax][int(i)]]
                     if i + 1 >= len(new_t):
@@ -510,7 +594,7 @@ class Classify(object):
                     dea = [lc_data[pbmask]['time'] < new_t[int(i+1)]]
 
                     ax1.errorbar(np.array(lc_data[pbmask]['time'])[dea], np.array(lc_data[pbmask]['flux'])[dea],
-                                 yerr=np.array(lc_data[pbmask]['fluxErr'])[dea], fmt=PB_MARKER[pb], label=pb,
+                                 yerr=np.array(lc_data[pbmask]['fluxErr'])[dea], fmt='o', label=pb,
                                  c=PB_COLOR[pb], lw=3, markersize=10)
 
                 for classnum, classname in enumerate(self.class_names):
